#ifndef _HAVE_INTERRUPT_H
#define _HAVE_INTERRUPT_H

#include "inttypes.h"

extern int32_t lock_counter;

#define	ARM_CPSR_INT_MASK (0x000000C0)
#define	ARM_CPSR_BIT_I (1 << 7)

inline static void DISABLE_INTERRUPTS() {
	asm volatile (
		/* get processor status in ip */
		"mrs ip,CPSR\n\t"
		/* disable interrupts by setting the flag */
		"orr ip,ip,%[mask]\n\t"
		/* write back new processor status */
		"msr CPSR_c,ip"
		:
		: [mask] "i" (ARM_CPSR_INT_MASK)
		: "ip", "memory", "cc"
		);
	lock_counter++;
}

inline static void ENABLE_INTERRUPTS() {
	lock_counter--;
	if(lock_counter <= 0){
		lock_counter = 0;
		asm volatile (
			/* get processor status in ip */
			"mrs ip,CPSR\n\t"
			/* enable interrupts by clearing the flag */
			"bic ip, ip, %[mask]\n\t"
			/* write back new processor status */
			"msr CPSR_c, ip"
			:
			: [mask] "i" (ARM_CPSR_BIT_I)
			: "ip", "memory", "cc"
		);
	}
}

#endif
